package MT::Plugin::CleanUp;
use strict;

use MT;
use MT::Plugin;

our $VERSION = "0.1";

@MT::Plugin::CleanUp::ISA = qw(MT::Plugin);

my $plugin = new MT::Plugin::CleanUp({
	name => 'CleanUp',
	version => $VERSION,
	description => 'Delete static files when the entry status is 1 or 4.',
	author_name => 'Junnama Noda',
	author_link => 'http://junnama.alfasado.net/online/',
});

MT->add_plugin($plugin);

MT->add_callback('MT::App::CMS::AppTemplateSource.list_blog', 1, $plugin, \&_CleanUp);
MT->add_callback('MT::App::CMS::AppTemplateOutput.list_blog', 1, $plugin, \&_AddStyle);

MT->add_plugin_action(	'blog',
							'../'.MT->config('AdminScript').'?cleanup=1&amp;',
							'Clean Up Files by Status'
						);

MT->add_plugin_action(	'list_entries',
							'../'.MT->config('AdminScript').'?cleanup=1&amp;',
							'Clean Up Files by Status'
						);

MT->add_callback('MT::App::CMS::AppTemplateParam.menu', 9, $plugin, \&_check_permission);

######################################### Mainroutine #########################################

sub _check_permission {
	my ($eh, $app, $param, $tmpl) = @_;
	my $blog = $app->blog;
	my $author = $app->user;
	use MT::Permission;
	my $perms = MT::Permission->load({
									blog_id => $blog->id,
									author_id => $author->id });
	unless ($perms->can_edit_all_posts) {
		@{$param->{plugin_action_loop}} = grep {
			$_->{'orig_link_text'} ne 'Clean up static files.'}
		@{$param->{plugin_action_loop}};
	}
	1;
}

my $counter = 0;

sub _CleanUp {
	my ($eh, $app, $tmpl_ref) = @_;
	my $query = $app->param;
	my $author = $app->user;
	my $mode = $query->param('cleanup');
	if ($mode) {
		$$tmpl_ref = &header;
		my $blog = $app->blog;
		my $all_flag = 0;
		unless (defined $blog) {
			unless ($author->is_superuser) {
				$app->redirect($app->mt_uri);
				return 0;
			}
			my $old = quotemeta('<TMPL_VAR NAME=BLOG_NAME ESCAPE=HTML>');
			$$tmpl_ref =~ s/$old/All blogs/s;
			$all_flag = 1;
		} else {
			my $perms = MT::Permission->load({
									blog_id => $blog->id,
									author_id => $author->id });
			unless ($perms->can_edit_all_posts) {
				$app->redirect($app->mt_uri);
				return 0;
			}
		}
		if ($mode == 1) {
			my $form = '<form name="cleanup" action='.$app->mt_uri.' method="post">'."\n";
			$form .= '<input type="hidden" name="cleanup" value="2" />'."\n";
			$$tmpl_ref .= $form;
			my $result;
			if (defined $blog) {
				my $blog_id = $blog->id;
				$form .= '<input type="hidden" name="blog_id" value="'.$blog_id.'" />'."\n";
				$result = '<ol>'.&check_static($blog, 1);
				$result .= &check_static($blog, 4);
				$result .= '</ol>';
			} else {
				$result = '<ol>'.&check_static('all', 1);
				$result .= &check_static('all', 4);
				$result .= '</ol>';
			}
			if ($counter) {
				$$tmpl_ref .= '<div class="message"><ul><li>';
				$$tmpl_ref .= $counter.' files found.</li></ul></div>';
				$$tmpl_ref .= $result.'<hr /><p style="text-align:center">'."\n";
				$$tmpl_ref .= '<input type="submit" value="Clean Up!" /></p>';
			} else {
				$$tmpl_ref .= '<div class="message"><ul><li>';
				$$tmpl_ref .= 'File to delete was not found.</li></ul></div>';
			}
			$$tmpl_ref .= '<hr /></form>';
			$$tmpl_ref .= &footer('-1');
		} elsif ($mode == 2) {
			my @ids = $query->param('id');
			use MT::Entry;
			use MT::Blog;
			my $err = 0;
			my $result = '';
			if (defined @ids) {
				$result .= '<ol>';
				foreach my $id (@ids) {
					my $entry;
					if (defined $blog) {
						$entry = MT::Entry->load({
								blog_id => $blog->id, id => $id });
					} else {
						$entry = MT::Entry->load({
											id => $id });
					}
					if (defined $entry) {
						my $permalink = $entry->permalink;
						if ($all_flag) {
							$blog = MT::Blog->load({ id => $entry->blog_id });
						}
						my $remove = &get_entry_path($blog, $permalink);
						if (-f $remove) {
							if (unlink($remove)) {
								$result .= '<li>ID: '.$id.' =&gt; removed.</li>';
							} else {
								$result .= '<li>ID: '.$id.' =&gt; failed.</li>';
								$err = 1;
							}
						} else {
							$result .= '<li>ID: '.$id.' =&gt; does not exist.</li>';
							$err = 1;
						}
					}
				}
				$result .= '</ol>';
				if ($err) {
					$$tmpl_ref .= '<div class="error-message">';
					$$tmpl_ref .= '<ul><li>Error occurred while clean up.</li></ul>';
					$$tmpl_ref .= '</div>';
				} else {
					$$tmpl_ref .= '<div class="message">';
					$$tmpl_ref .= '<ul><li>Clean up was successful.</li></ul>';
					$$tmpl_ref .= '</div>';
				}
				$$tmpl_ref .= $result.'<hr />';
			}
			$$tmpl_ref .= &footer('-2');
		}
	}
}

sub _AddStyle {
	my ($eh, $app, $tmpl_ref) = @_;
	my $query = $app->param;
	my $mode = $query->param('cleanup');
	if ($mode) {
		my ($eh, $app, $tmpl_ref) = @_;
		my $old = '<\/head>';
		my $new = '<style>ul li{font-size:110%}ol li{font-size:93%}</style></head>';
		$$tmpl_ref =~ s/$old/$new/;
	}
}

######################################### Subroutine ##########################################

sub check_static {
	my ($blog, $status) = @_;
	use MT::Entry;
	my $res = '';
	my $iter;
	use MT::Blog;
	if ($blog ne 'all') {
		$iter = MT::Entry->load_iter({blog_id => $blog->id,
											 status => $status,
											});
	} else {
		$iter = MT::Entry->load_iter({status => $status,
											});
	}
	if (defined $iter) {
		while (my $entry = $iter->()) {
			my $permalink = $entry->permalink;
			if ($blog eq 'all') {
				$blog = MT::Blog->load({ id => $entry->blog_id });
			}
			my $remove = &get_entry_path($blog, $permalink);
			if (-f $remove) {
				my $line = '<li><label><input type="checkbox" name="id" value="';
				$line .= $entry->id.'" checked="checked" /> <a href="'.$permalink.'" target="view">';
				$line .= $entry->title.'</a></label></li>';
				$res .= $line;
				$counter++;
			}
		}
	}
	return $res;
}

sub get_entry_path {
	my ($blog, $permalink) = @_;
	my $arcive_path = $blog->archive_path;
	$arcive_path =~ s/\/$//;
	my $archive_url = $blog->archive_url;
	$archive_url =~ s/\/$//;
	$archive_url = quotemeta($archive_url);
	$permalink =~ s/^$archive_url/$arcive_path/;
	return $permalink;
}

########################################## Templates ###########################################

sub header {
	return <<HTML;

<TMPL_INCLUDE NAME="header-popup.tmpl">
<h2><span class="weblog-title-highlight">
<TMPL_VAR NAME=BLOG_NAME ESCAPE=HTML>:</span> Clean up static files.
</h2>
<div>

HTML
}

sub footer {
	my $history = shift;
	return <<HTML;

</div>
<div style="text-align:center;">
<input type="button" onclick="history.go($history)" value="<MT_TRANS phrase="Back">" />
<input type="button" onclick="window.close()" value="<MT_TRANS phrase="Close">" />
<TMPL_INCLUDE NAME="footer-popup.tmpl">
HTML
}

1;